/**
 * The MIT License (MIT)
 *
 * Copyright (C) 2013 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.apidesign.bck2brwsr.demo.serverside;

import net.java.html.json.ComputedProperty;
import net.java.html.json.Context;
import net.java.html.json.Function;
import net.java.html.json.Model;
import net.java.html.json.OnReceive;
import net.java.html.json.Property;

/**
 *
 * @author Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 */
@Model(className = "ChatModel", properties = {
    @Property(name = "user", type = String.class),
    @Property(name = "text", type = String.class),
    @Property(name = "msgs", type = Message.class, array = true)
})
class ChatClient {
    @ComputedProperty
    static boolean sendEnabled(String user, String text) {
        boolean res = user != null && text != null && !user.isEmpty() && !text.isEmpty();
        try {
            if (true) throw new IllegalStateException("Query for msg '" + text + "' by user " + user + " res: " + res);
        } catch (Exception ex) {
            
        }
        return res;
    }
    
    @Function
    static void submit(ChatModel m) {
        if (!sendEnabled(m.getUser(), m.getText())) {
            return;
        }
        Message msg = new Message(CNTX);
        msg.setComment(m.getText());
        msg.setUser(m.getUser());
        m.getMsgs().add(msg);
    }
    
    @OnReceive(url = "/chat?since=0")
    static void initialRead(ChatModel m, Query q) {
        m.getMsgs().clear();
        m.getMsgs().addAll(q.getMessages());
    }
    
    static final Context CNTX = Context.findDefault(ChatClient.class);
    static {
        ChatModel chm = new ChatModel(CNTX);
        Message m = new Message(CNTX);
        m.setComment("Waiting for messages from the server...");
        m.setUser("system");
        chm.getMsgs().add(m);
        chm.applyBindings();
        chm.initialRead();
    }
}
